#version 330 core
const int MAX_LIGHTS = 10;

layout(location = 0) in vec3 position;
in vec3 normal;
in vec3 tangent;
in vec3 color;
in vec2 uv;

// Output data ; will be interpolated for each fragment.
out vec2 fUVs;
out vec3 Position_worldspace;
out vec3 Normal_worldspace;
//out vec3 Normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace[MAX_LIGHTS];
out vec4 ShadowCoord;
out vec3 tangentout;

// Values that stay constant for the whole mesh.
uniform mat4 dVP;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat4 modelMatrix;

uniform int NUM_LIGHTS;
uniform int lightType[MAX_LIGHTS];
uniform	vec3 lightPower[MAX_LIGHTS];
uniform vec3 lightPosition[MAX_LIGHTS];
uniform vec3 lightDirection[MAX_LIGHTS];
uniform	float cutoff[MAX_LIGHTS];
uniform	float exponent[MAX_LIGHTS];

mat4 bias = mat4(
		0.5, 0.0, 0.0, 0.0, 
		0.0, 0.5, 0.0, 0.0,
		0.0, 0.0, 0.5, 0.0,
		0.5, 0.5, 0.5, 1.0
		);

void main(){

	gl_Position =  projectionMatrix* viewMatrix *   modelMatrix * vec4(position,1);
	Position_worldspace = (modelMatrix * vec4(position,1)).xyz;
	
	
	ShadowCoord = bias * dVP * modelMatrix * vec4(position,1);

	// Vector that goes from the vertex to the camera, in camera space.
	vec3 vertexPosition_cameraspace = ( viewMatrix * modelMatrix * vec4(position,1)).xyz;
	EyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;

	//Normal_worldspace = normalize(mat3(modelMatrix)*normal);

	 for(int i = 0; i < NUM_LIGHTS; ++i){
			vec3 LightPosition_cameraspace = ( viewMatrix * vec4(lightPosition[i],1)).xyz;
			LightDirection_cameraspace[i] = LightPosition_cameraspace + EyeDirection_cameraspace;
	}
	
	Normal_worldspace = normalize(mat3(modelMatrix)*normal);
	tangentout = (modelMatrix*vec4(tangent,0.0)).xyz;

	//Normal_cameraspace = ( viewMatrix * modelMatrix * vec4(normal,0)).xyz; // Only correct if modelMatrix does not scale the model ! Use its inverse transpose if not.
	
	fUVs = uv;
}